@@ -1,7 +1,4 @@
-from __future__ import print_function, division
-
 from sympy.combinatorics.permutations import Permutation
-from sympy.core.compatibility import range
 from sympy.core.symbol import symbols
 from sympy.matrices import Matrix
 from sympy.utilities.iterables import variations, rotate_left
@@ -15,7 +12,6 @@ def symmetric(n):
     ========
 
     >>> from sympy.combinatorics.permutations import Permutation
-    >>> Permutation.print_cyclic = True
     >>> from sympy.combinatorics.generators import symmetric
     >>> list(symmetric(3))
     [(2), (1 2), (2)(0 1), (0 1 2), (0 2 1), (0 2)]
@@ -32,7 +28,6 @@ def cyclic(n):
     ========
 
     >>> from sympy.combinatorics.permutations import Permutation
-    >>> Permutation.print_cyclic = True
     >>> from sympy.combinatorics.generators import cyclic
     >>> list(cyclic(5))
     [(4), (0 1 2 3 4), (0 2 4 1 3),
@@ -57,7 +52,6 @@ def alternating(n):
     ========
 
     >>> from sympy.combinatorics.permutations import Permutation
-    >>> Permutation.print_cyclic = True
     >>> from sympy.combinatorics.generators import alternating
     >>> list(alternating(3))
     [(2), (0 1 2), (0 2 1)]
@@ -80,7 +74,6 @@ def dihedral(n):
     ========
 
     >>> from sympy.combinatorics.permutations import Permutation
-    >>> Permutation.print_cyclic = True
     >>> from sympy.combinatorics.generators import dihedral
     >>> list(dihedral(3))
     [(2), (0 2), (0 1 2), (1 2), (0 2 1), (2)(0 1)]
@@ -186,9 +179,9 @@ def rubik(n):
                 cw(F)
             i += 1
             temp = getr(L, i)
-            setr(L, i, list((getu(D, i))))
+            setr(L, i, list(getu(D, i)))
             setu(D, i, list(reversed(getl(R, i))))
-            setl(R, i, list((getd(U, i))))
+            setl(R, i, list(getd(U, i)))
             setd(U, i, list(reversed(temp)))
             i -= 1
 
