@@ -1,9 +1,7 @@
 """Various algorithms for helping identifying numbers and sequences."""
-from __future__ import print_function, division
 
 from sympy.utilities import public
 
-from sympy.core.compatibility import range
 from sympy.core import Function, Symbol
 from sympy.core.numbers import Zero
 from sympy import (sympify, floor, lcm, denom, Integer, Rational,
@@ -38,7 +36,7 @@ def find_simple_recurrence_vector(l):
     >>> find_simple_recurrence_vector([fibonacci(k) for k in range(12)])
     [1, -1, -1]
 
-    See also
+    See Also
     ========
 
     See the function sympy.concrete.guess.find_simple_recurrence which is more
@@ -132,7 +130,7 @@ def rationalize(x, maxcoeff=10000):
     >>> rationalize(pi, maxcoeff = 250)
     355/113
 
-    See also
+    See Also
     ========
     Several other methods can approximate a real number as a rational, like:
 
@@ -182,9 +180,11 @@ def guess_generating_function_rational(v, X=Symbol('x')):
     >>> guess_generating_function_rational(l)
     (3*x + 5)/(-x**2 - x + 1)
 
-    See also
+    See Also
     ========
-    See function sympy.series.approximants and mpmath.pade
+
+    sympy.series.approximants
+    mpmath.pade
 
     """
     #   a) compute the denominator as q
@@ -237,7 +237,7 @@ def guess_generating_function(v, X=Symbol('x'), types=['all'], maxsqrtn=2):
 
     >>> from sympy.concrete.guess import guess_generating_function as ggf
     >>> ggf([k+1 for k in range(12)], types=['ogf', 'lgf', 'hlgf'])
-    {'hlgf': 1/(-x + 1), 'lgf': 1/(x + 1), 'ogf': 1/(x**2 - 2*x + 1)}
+    {'hlgf': 1/(1 - x), 'lgf': 1/(x + 1), 'ogf': 1/(x**2 - 2*x + 1)}
 
     >>> from sympy import sympify
     >>> l = sympify("[3/2, 11/2, 0, -121/2, -363/2, 121]")
@@ -250,7 +250,7 @@ def guess_generating_function(v, X=Symbol('x'), types=['all'], maxsqrtn=2):
 
     >>> from sympy import simplify, factorial
     >>> ggf([factorial(k) for k in range(12)], types=['ogf', 'egf', 'lgf'])
-    {'egf': 1/(-x + 1)}
+    {'egf': 1/(1 - x)}
 
     >>> ggf([k+1 for k in range(12)], types=['egf'])
     {'egf': (x + 1)*exp(x), 'lgdegf': (x + 2)/(x + 1)}
@@ -264,8 +264,9 @@ def guess_generating_function(v, X=Symbol('x'), types=['all'], maxsqrtn=2):
 
     References
     ==========
-    "Concrete Mathematics", R.L. Graham, D.E. Knuth, O. Patashnik
-    https://oeis.org/wiki/Generating_functions
+
+    .. [1] "Concrete Mathematics", R.L. Graham, D.E. Knuth, O. Patashnik
+    .. [2] https://oeis.org/wiki/Generating_functions
 
     """
     # List of all types of all g.f. known by the algorithm
@@ -415,6 +416,9 @@ def guess(l, all=False, evaluate=True, niter=2, variables=None):
     the first symbols will be used); in this case, the main variable will be
     the first symbol in the list.
 
+    Examples
+    ========
+
     >>> from sympy.concrete.guess import guess
     >>> guess([1,2,6,24,120], evaluate=False)
     [Product(i1 + 1, (i1, 1, i0 - 1))]
@@ -432,7 +436,7 @@ def guess(l, all=False, evaluate=True, niter=2, variables=None):
     myprod = product if evaluate else Product
     g = []
     res = []
-    if variables == None:
+    if variables is None:
         symb = symbols('i:'+str(niter))
     else:
         symb = variables
