@@ -1,12 +1,11 @@
 """sympify -- convert objects SymPy internal format"""
 
-from __future__ import print_function, division
+from typing import Dict, Type, Callable, Any
 
 from inspect import getmro
 
-from .core import all_classes as sympy_classes
-from .compatibility import iterable, string_types, range
-from .evaluate import global_evaluate
+from .compatibility import iterable
+from .parameters import global_parameters
 
 
 class SympifyError(ValueError):
@@ -22,9 +21,12 @@ class SympifyError(ValueError):
             "raised:\n%s: %s" % (self.expr, self.base_exc.__class__.__name__,
             str(self.base_exc)))
 
-converter = {}  # See sympify docstring.
 
-class CantSympify(object):
+# See sympify docstring.
+converter = {}  # type: Dict[Type[Any], Callable[[Any], Basic]]
+
+
+class CantSympify:
     """
     Mix in this trait to a class to disallow sympification of its instances.
 
@@ -51,14 +53,26 @@ class CantSympify(object):
     pass
 
 
-def _convert_numpy_types(a):
+def _is_numpy_instance(a):
     """
-    Converts a numpy datatype input to an appropriate sympy type.
+    Checks if an object is an instance of a type from the numpy module.
+    """
+    # This check avoids unnecessarily importing NumPy.  We check the whole
+    # __mro__ in case any base type is a numpy type.
+    return any(type_.__module__ == 'numpy'
+               for type_ in type(a).__mro__)
+
+
+def _convert_numpy_types(a, **sympify_args):
+    """
+    Converts a numpy datatype input to an appropriate SymPy type.
     """
     import numpy as np
     if not isinstance(a, np.floating):
-        func = converter[complex] if np.iscomplex(a) else sympify
-        return func(np.asscalar(a))
+        if np.iscomplex(a):
+            return converter[complex](a.item())
+        else:
+            return sympify(a.item(), **sympify_args)
     else:
         try:
             from sympy.core.numbers import Float
@@ -83,11 +97,11 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     with SAGE.
 
     It currently accepts as arguments:
-       - any object defined in sympy
+       - any object defined in SymPy
        - standard numeric python types: int, long, float, Decimal
        - strings (like "0.09" or "2e-19")
        - booleans, including ``None`` (will leave ``None`` unchanged)
-       - lists, sets or tuples containing any of the above
+       - dict, lists, sets or tuples containing any of the above
 
     .. warning::
         Note that this function uses ``eval``, and thus shouldn't be used on
@@ -240,6 +254,9 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     Notes
     =====
 
+    The keywords ``rational`` and ``convert_xor`` are only used
+    when the input is a string.
+
     Sometimes autosimplification during sympification results in expressions
     that are very different in structure than what was entered. Until such
     autosimplification is no longer done, the ``kernS`` function might be of
@@ -257,67 +274,87 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
     -2*(-(-x + 1/x)/(x*(x - 1/x)**2) - 1/(x*(x - 1/x))) - 1
 
     """
-    if evaluate is None:
-        if global_evaluate[0] is False:
-            evaluate = global_evaluate[0]
-        else:
-            evaluate = True
-    try:
-        if a in sympy_classes:
-            return a
-    except TypeError: # Type of a is unhashable
-        pass
-    try:
-        cls = a.__class__
-    except AttributeError:  # a is probably an old-style class object
-        cls = type(a)
-    if cls in sympy_classes:
+    is_sympy = getattr(a, '__sympy__', None)
+    if is_sympy is not None:
         return a
+
+    if isinstance(a, CantSympify):
+        raise SympifyError(a)
+    cls = getattr(a, "__class__", None)
+    if cls is None:
+        cls = type(a)  # Probably an old-style class
+    conv = converter.get(cls, None)
+    if conv is not None:
+        return conv(a)
+
+    for superclass in getmro(cls):
+        try:
+            return converter[superclass](a)
+        except KeyError:
+            continue
+
     if cls is type(None):
         if strict:
             raise SympifyError(a)
         else:
             return a
 
+    if evaluate is None:
+        evaluate = global_parameters.evaluate
+
     # Support for basic numpy datatypes
-    # Note that this check exists to avoid importing NumPy when not necessary
-    if type(a).__module__ == 'numpy':
+    if _is_numpy_instance(a):
         import numpy as np
         if np.isscalar(a):
-            return _convert_numpy_types(a)
+            return _convert_numpy_types(a, locals=locals,
+                convert_xor=convert_xor, strict=strict, rational=rational,
+                evaluate=evaluate)
 
-    try:
-        return converter[cls](a)
-    except KeyError:
-        for superclass in getmro(cls):
-            try:
-                return converter[superclass](a)
-            except KeyError:
-                continue
-
-    if isinstance(a, CantSympify):
-        raise SympifyError(a)
-
-    try:
-        return a._sympy_()
-    except AttributeError:
-        pass
+    _sympy_ = getattr(a, "_sympy_", None)
+    if _sympy_ is not None:
+        try:
+            return a._sympy_()
+        # XXX: Catches AttributeError: 'SympyConverter' object has no
+        # attribute 'tuple'
+        # This is probably a bug somewhere but for now we catch it here.
+        except AttributeError:
+            pass
 
     if not strict:
         # Put numpy array conversion _before_ float/int, see
         # <https://github.com/sympy/sympy/issues/13924>.
-        try:
-            from ..tensor.array import Array
-            return Array(a.flat, a.shape)  # works with e.g. NumPy arrays
-        except AttributeError:
-            pass
+        flat = getattr(a, "flat", None)
+        if flat is not None:
+            shape = getattr(a, "shape", None)
+            if shape is not None:
+                from ..tensor.array import Array
+                return Array(a.flat, a.shape)  # works with e.g. NumPy arrays
 
-    if not isinstance(a, string_types):
-        for coerce in (float, int):
-            try:
-                return sympify(coerce(a))
-            except (TypeError, ValueError, AttributeError, SympifyError):
-                continue
+    if not isinstance(a, str):
+        if _is_numpy_instance(a):
+            import numpy as np
+            assert not isinstance(a, np.number)
+            if isinstance(a, np.ndarray):
+                # Scalar arrays (those with zero dimensions) have sympify
+                # called on the scalar element.
+                if a.ndim == 0:
+                    try:
+                        return sympify(a.item(),
+                                       locals=locals,
+                                       convert_xor=convert_xor,
+                                       strict=strict,
+                                       rational=rational,
+                                       evaluate=evaluate)
+                    except SympifyError:
+                        pass
+        else:
+            # float and int can coerce size-one numpy arrays to their lone
+            # element.  See issue https://github.com/numpy/numpy/issues/10404.
+            for coerce in (float, int):
+                try:
+                    return sympify(coerce(a))
+                except (TypeError, ValueError, AttributeError, SympifyError):
+                    continue
 
     if strict:
         raise SympifyError(a)
@@ -337,19 +374,20 @@ def sympify(a, locals=None, convert_xor=True, strict=False, rational=False,
             # Not all iterables are rebuildable with their type.
             pass
 
-    # At this point we were given an arbitrary expression
-    # which does not inherit from Basic and doesn't implement
-    # _sympy_ (which is a canonical and robust way to convert
-    # anything to SymPy expression).
-    #
-    # As a last chance, we try to take "a"'s normal form via unicode()
-    # and try to parse it. If it fails, then we have no luck and
-    # return an exception
-    try:
-        from .compatibility import unicode
-        a = unicode(a)
-    except Exception as exc:
-        raise SympifyError(a, exc)
+    if not isinstance(a, str):
+        try:
+            a = str(a)
+        except Exception as exc:
+            raise SympifyError(a, exc)
+        from sympy.utilities.exceptions import SymPyDeprecationWarning
+        SymPyDeprecationWarning(
+            feature="String fallback in sympify",
+            useinstead= \
+                'sympify(str(obj)) or ' + \
+                'sympy.core.sympify.converter or obj._sympy_',
+            issue=18066,
+            deprecated_since_version='1.6'
+        ).warn()
 
     from sympy.parsing.sympy_parser import (parse_expr, TokenError,
                                             standard_transformations)
@@ -481,7 +519,7 @@ def kernS(s):
         try:
             expr = sympify(s)
             break
-        except:  # the kern might cause unknown errors, so use bare except
+        except TypeError:  # the kern might cause unknown errors...
             if hit:
                 s = olds  # maybe it didn't like the kern; use un-kerned s
                 hit = False
@@ -501,3 +539,7 @@ def kernS(s):
     expr = _clear(expr)
     # hope that kern is not there anymore
     return expr
+
+
+# Avoid circular import
+from .basic import Basic
