@@ -1,10 +1,7 @@
-from __future__ import print_function, division
-
 from sympy.core.add import Add
 from sympy.core.basic import sympify, cacheit
-from sympy.core.compatibility import range
-from sympy.core.function import Function, ArgumentIndexError
-from sympy.core.logic import fuzzy_not, fuzzy_or
+from sympy.core.function import Function, ArgumentIndexError, expand_mul
+from sympy.core.logic import fuzzy_not, fuzzy_or, FuzzyBool
 from sympy.core.numbers import igcdex, Rational, pi
 from sympy.core.relational import Ne
 from sympy.core.singleton import S
@@ -66,7 +63,7 @@ class TrigonometricFunction(Function):
         return (re, im)
 
     def _period(self, general_period, symbol=None):
-        f = self.args[0]
+        f = expand_mul(self.args[0])
         if symbol is None:
             symbol = tuple(f.free_symbols)[0]
 
@@ -120,7 +117,7 @@ def _peeloff_pi(arg):
     else:
         return arg, S.Zero
 
-    m1 = (K % S.Half) * S.Pi
+    m1 = (K % S.Half)*S.Pi
     m2 = K*S.Pi - m1
     return arg - m2, m2
 
@@ -301,7 +298,7 @@ class sin(TrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return S.ImaginaryUnit * sinh(i_coeff)
+            return S.ImaginaryUnit*sinh(i_coeff)
 
         pi_coeff = _pi_coeff(arg)
         if pi_coeff is not None:
@@ -349,11 +346,11 @@ class sin(TrigonometricFunction):
 
         if isinstance(arg, atan):
             x = arg.args[0]
-            return x / sqrt(1 + x**2)
+            return x/sqrt(1 + x**2)
 
         if isinstance(arg, atan2):
             y, x = arg.args
-            return y / sqrt(x**2 + y**2)
+            return y/sqrt(x**2 + y**2)
 
         if isinstance(arg, acos):
             x = arg.args[0]
@@ -361,15 +358,15 @@ class sin(TrigonometricFunction):
 
         if isinstance(arg, acot):
             x = arg.args[0]
-            return 1 / (sqrt(1 + 1 / x**2) * x)
+            return 1/(sqrt(1 + 1/x**2)*x)
 
         if isinstance(arg, acsc):
             x = arg.args[0]
-            return 1 / x
+            return 1/x
 
         if isinstance(arg, asec):
             x = arg.args[0]
-            return sqrt(1 - 1 / x**2)
+            return sqrt(1 - 1/x**2)
 
     @staticmethod
     @cacheit
@@ -381,24 +378,24 @@ class sin(TrigonometricFunction):
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
-                return -p * x**2 / (n*(n - 1))
+                return -p*x**2/(n*(n - 1))
             else:
-                return (-1)**(n//2) * x**(n)/factorial(n)
+                return (-1)**(n//2)*x**(n)/factorial(n)
 
     def _eval_rewrite_as_exp(self, arg, **kwargs):
         I = S.ImaginaryUnit
         if isinstance(arg, TrigonometricFunction) or isinstance(arg, HyperbolicFunction):
             arg = arg.func(arg.args[0]).rewrite(exp)
-        return (exp(arg*I) - exp(-arg*I)) / (2*I)
+        return (exp(arg*I) - exp(-arg*I))/(2*I)
 
     def _eval_rewrite_as_Pow(self, arg, **kwargs):
         if isinstance(arg, log):
             I = S.ImaginaryUnit
             x = arg.args[0]
-            return I*x**-I / 2 - I*x**I /2
+            return I*x**-I/2 - I*x**I /2
 
     def _eval_rewrite_as_cos(self, arg, **kwargs):
-        return cos(arg - S.Pi / 2, evaluate=False)
+        return cos(arg - S.Pi/2, evaluate=False)
 
     def _eval_rewrite_as_tan(self, arg, **kwargs):
         tan_half = tan(S.Half*arg)
@@ -421,7 +418,7 @@ class sin(TrigonometricFunction):
         return 1/csc(arg)
 
     def _eval_rewrite_as_sec(self, arg, **kwargs):
-        return 1 / sec(arg - S.Pi / 2, evaluate=False)
+        return 1/sec(arg - S.Pi/2, evaluate=False)
 
     def _eval_rewrite_as_sinc(self, arg, **kwargs):
         return arg*sinc(arg)
@@ -666,11 +663,11 @@ class cos(TrigonometricFunction):
 
         if isinstance(arg, atan):
             x = arg.args[0]
-            return 1 / sqrt(1 + x**2)
+            return 1/sqrt(1 + x**2)
 
         if isinstance(arg, atan2):
             y, x = arg.args
-            return x / sqrt(x**2 + y**2)
+            return x/sqrt(x**2 + y**2)
 
         if isinstance(arg, asin):
             x = arg.args[0]
@@ -678,15 +675,15 @@ class cos(TrigonometricFunction):
 
         if isinstance(arg, acot):
             x = arg.args[0]
-            return 1 / sqrt(1 + 1 / x**2)
+            return 1/sqrt(1 + 1/x**2)
 
         if isinstance(arg, acsc):
             x = arg.args[0]
-            return sqrt(1 - 1 / x**2)
+            return sqrt(1 - 1/x**2)
 
         if isinstance(arg, asec):
             x = arg.args[0]
-            return 1 / x
+            return 1/x
 
     @staticmethod
     @cacheit
@@ -698,7 +695,7 @@ class cos(TrigonometricFunction):
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
-                return -p * x**2 / (n*(n - 1))
+                return -p*x**2/(n*(n - 1))
             else:
                 return (-1)**(n//2)*x**(n)/factorial(n)
 
@@ -706,7 +703,7 @@ class cos(TrigonometricFunction):
         I = S.ImaginaryUnit
         if isinstance(arg, TrigonometricFunction) or isinstance(arg, HyperbolicFunction):
             arg = arg.func(arg.args[0]).rewrite(exp)
-        return (exp(arg*I) + exp(-arg*I)) / 2
+        return (exp(arg*I) + exp(-arg*I))/2
 
     def _eval_rewrite_as_Pow(self, arg, **kwargs):
         if isinstance(arg, log):
@@ -715,7 +712,7 @@ class cos(TrigonometricFunction):
             return x**I/2 + x**-I/2
 
     def _eval_rewrite_as_sin(self, arg, **kwargs):
-        return sin(arg + S.Pi / 2, evaluate=False)
+        return sin(arg + S.Pi/2, evaluate=False)
 
     def _eval_rewrite_as_tan(self, arg, **kwargs):
         tan_half = tan(S.Half*arg)**2
@@ -873,7 +870,7 @@ class cos(TrigonometricFunction):
         return 1/sec(arg)
 
     def _eval_rewrite_as_csc(self, arg, **kwargs):
-        return 1 / sec(arg).rewrite(csc)
+        return 1/sec(arg).rewrite(csc)
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
@@ -1012,7 +1009,7 @@ class tan(TrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return S.ImaginaryUnit * tanh(i_coeff)
+            return S.ImaginaryUnit*tanh(i_coeff)
 
         pi_coeff = _pi_coeff(arg, 2)
         if pi_coeff is not None:
@@ -1036,7 +1033,7 @@ class tan(TrigonometricFunction):
                     4: sqrt(5 + 2*sqrt(5))
                     }
                 if q == 5 or q == 10:
-                    n = 10 * p / q
+                    n = 10*p/q
                     if n > 5:
                         n = 10 - n
                         return -table10[n]
@@ -1098,23 +1095,23 @@ class tan(TrigonometricFunction):
 
         if isinstance(arg, asin):
             x = arg.args[0]
-            return x / sqrt(1 - x**2)
+            return x/sqrt(1 - x**2)
 
         if isinstance(arg, acos):
             x = arg.args[0]
-            return sqrt(1 - x**2) / x
+            return sqrt(1 - x**2)/x
 
         if isinstance(arg, acot):
             x = arg.args[0]
-            return 1 / x
+            return 1/x
 
         if isinstance(arg, acsc):
             x = arg.args[0]
-            return 1 / (sqrt(1 - 1 / x**2) * x)
+            return 1/(sqrt(1 - 1/x**2)*x)
 
         if isinstance(arg, asec):
             x = arg.args[0]
-            return sqrt(1 - 1 / x**2) * x
+            return sqrt(1 - 1/x**2)*x
 
     @staticmethod
     @cacheit
@@ -1130,7 +1127,7 @@ class tan(TrigonometricFunction):
             B = bernoulli(n + 1)
             F = factorial(n + 1)
 
-            return (-1)**a * b*(b - 1) * B/F * x**n
+            return (-1)**a*b*(b - 1)*B/F*x**n
 
     def _eval_nseries(self, x, n, logx):
         i = self.args[0].limit(x, 0)*2/S.Pi
@@ -1195,7 +1192,7 @@ class tan(TrigonometricFunction):
         return 2*sin(x)**2/sin(2*x)
 
     def _eval_rewrite_as_cos(self, x, **kwargs):
-        return cos(x - S.Pi / 2, evaluate=False) / cos(x)
+        return cos(x - S.Pi/2, evaluate=False)/cos(x)
 
     def _eval_rewrite_as_sincos(self, arg, **kwargs):
         return sin(arg)/cos(arg)
@@ -1206,12 +1203,12 @@ class tan(TrigonometricFunction):
     def _eval_rewrite_as_sec(self, arg, **kwargs):
         sin_in_sec_form = sin(arg).rewrite(sec)
         cos_in_sec_form = cos(arg).rewrite(sec)
-        return sin_in_sec_form / cos_in_sec_form
+        return sin_in_sec_form/cos_in_sec_form
 
     def _eval_rewrite_as_csc(self, arg, **kwargs):
         sin_in_csc_form = sin(arg).rewrite(csc)
         cos_in_csc_form = cos(arg).rewrite(csc)
-        return sin_in_csc_form / cos_in_csc_form
+        return sin_in_csc_form/cos_in_csc_form
 
     def _eval_rewrite_as_pow(self, arg, **kwargs):
         y = self.rewrite(cos).rewrite(pow)
@@ -1240,13 +1237,13 @@ class tan(TrigonometricFunction):
 
     def _eval_is_real(self):
         arg = self.args[0]
-        if arg.is_real and (arg / pi - S.Half).is_integer is False:
+        if arg.is_real and (arg/pi - S.Half).is_integer is False:
             return True
 
     def _eval_is_finite(self):
         arg = self.args[0]
 
-        if arg.is_real and (arg / pi - S.Half).is_integer is False:
+        if arg.is_real and (arg/pi - S.Half).is_integer is False:
             return True
 
         if arg.is_imaginary:
@@ -1260,7 +1257,7 @@ class tan(TrigonometricFunction):
     def _eval_is_complex(self):
         arg = self.args[0]
 
-        if arg.is_real and (arg / pi - S.Half).is_integer is False:
+        if arg.is_real and (arg/pi - S.Half).is_integer is False:
             return True
 
 
@@ -1337,7 +1334,7 @@ class cot(TrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return -S.ImaginaryUnit * coth(i_coeff)
+            return -S.ImaginaryUnit*coth(i_coeff)
 
         pi_coeff = _pi_coeff(arg, 2)
         if pi_coeff is not None:
@@ -1384,7 +1381,7 @@ class cot(TrigonometricFunction):
                         and not isinstance(sresult, cos):
                     if sresult == 0:
                         return S.ComplexInfinity
-                    return cresult / sresult
+                    return cresult/sresult
                 if narg != arg:
                     return cls(narg)
 
@@ -1405,7 +1402,7 @@ class cot(TrigonometricFunction):
 
         if isinstance(arg, atan):
             x = arg.args[0]
-            return 1 / x
+            return 1/x
 
         if isinstance(arg, atan2):
             y, x = arg.args
@@ -1413,26 +1410,26 @@ class cot(TrigonometricFunction):
 
         if isinstance(arg, asin):
             x = arg.args[0]
-            return sqrt(1 - x**2) / x
+            return sqrt(1 - x**2)/x
 
         if isinstance(arg, acos):
             x = arg.args[0]
-            return x / sqrt(1 - x**2)
+            return x/sqrt(1 - x**2)
 
         if isinstance(arg, acsc):
             x = arg.args[0]
-            return sqrt(1 - 1 / x**2) * x
+            return sqrt(1 - 1/x**2)*x
 
         if isinstance(arg, asec):
             x = arg.args[0]
-            return 1 / (sqrt(1 - 1 / x**2) * x)
+            return 1/(sqrt(1 - 1/x**2)*x)
 
     @staticmethod
     @cacheit
     def taylor_term(n, x, *previous_terms):
         from sympy import bernoulli
         if n == 0:
-            return 1 / sympify(x)
+            return 1/sympify(x)
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
@@ -1441,7 +1438,7 @@ class cot(TrigonometricFunction):
             B = bernoulli(n + 1)
             F = factorial(n + 1)
 
-            return (-1)**((n + 1)//2) * 2**(n + 1) * B/F * x**n
+            return (-1)**((n + 1)//2)*2**(n + 1)*B/F*x**n
 
     def _eval_nseries(self, x, n, logx):
         i = self.args[0].limit(x, 0)/S.Pi
@@ -1477,7 +1474,7 @@ class cot(TrigonometricFunction):
         return sin(2*x)/(2*(sin(x)**2))
 
     def _eval_rewrite_as_cos(self, x, **kwargs):
-        return cos(x) / cos(x - S.Pi / 2, evaluate=False)
+        return cos(x)/cos(x - S.Pi/2, evaluate=False)
 
     def _eval_rewrite_as_sincos(self, arg, **kwargs):
         return cos(arg)/sin(arg)
@@ -1488,12 +1485,12 @@ class cot(TrigonometricFunction):
     def _eval_rewrite_as_sec(self, arg, **kwargs):
         cos_in_sec_form = cos(arg).rewrite(sec)
         sin_in_sec_form = sin(arg).rewrite(sec)
-        return cos_in_sec_form / sin_in_sec_form
+        return cos_in_sec_form/sin_in_sec_form
 
     def _eval_rewrite_as_csc(self, arg, **kwargs):
         cos_in_csc_form = cos(arg).rewrite(csc)
         sin_in_csc_form = sin(arg).rewrite(csc)
-        return cos_in_csc_form / sin_in_csc_form
+        return cos_in_csc_form/sin_in_csc_form
 
     def _eval_rewrite_as_pow(self, arg, **kwargs):
         y = self.rewrite(cos).rewrite(pow)
@@ -1561,7 +1558,7 @@ class cot(TrigonometricFunction):
 
     def _eval_is_complex(self):
         arg = self.args[0]
-        if arg.is_real and (arg / pi).is_integer is False:
+        if arg.is_real and (arg/pi).is_integer is False:
             return True
 
     def _eval_subs(self, old, new):
@@ -1580,8 +1577,10 @@ class ReciprocalTrigonometricFunction(TrigonometricFunction):
     # _is_even and _is_odd are used for correct evaluation of csc(-x), sec(-x)
     # TODO refactor into TrigonometricFunction common parts of
     # trigonometric functions eval() like even/odd, func(x+2*k*pi), etc.
-    _is_even = None  # optional, to be defined in subclass
-    _is_odd = None   # optional, to be defined in subclass
+
+    # optional, to be defined in subclasses:
+    _is_even = None  # type: FuzzyBool
+    _is_odd = None  # type: FuzzyBool
 
     @classmethod
     def eval(cls, arg):
@@ -1639,7 +1638,7 @@ class ReciprocalTrigonometricFunction(TrigonometricFunction):
             return 1/t
 
     def _period(self, symbol):
-        f = self.args[0]
+        f = expand_mul(self.args[0])
         return self._reciprocal_of(f).period(symbol)
 
     def fdiff(self, argindex=1):
@@ -1742,13 +1741,13 @@ class sec(ReciprocalTrigonometricFunction):
         return sin(arg)/(cos(arg)*sin(arg))
 
     def _eval_rewrite_as_sin(self, arg, **kwargs):
-        return (1 / cos(arg).rewrite(sin))
+        return (1/cos(arg).rewrite(sin))
 
     def _eval_rewrite_as_tan(self, arg, **kwargs):
-        return (1 / cos(arg).rewrite(tan))
+        return (1/cos(arg).rewrite(tan))
 
     def _eval_rewrite_as_csc(self, arg, **kwargs):
-        return csc(pi / 2 - arg, evaluate=False)
+        return csc(pi/2 - arg, evaluate=False)
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -1759,7 +1758,7 @@ class sec(ReciprocalTrigonometricFunction):
     def _eval_is_complex(self):
         arg = self.args[0]
 
-        if arg.is_complex and (arg / pi - S.Half).is_integer is False:
+        if arg.is_complex and (arg/pi - S.Half).is_integer is False:
             return True
 
     @staticmethod
@@ -1829,13 +1828,13 @@ class csc(ReciprocalTrigonometricFunction):
         return (1 + cot_half**2)/(2*cot_half)
 
     def _eval_rewrite_as_cos(self, arg, **kwargs):
-        return 1 / sin(arg).rewrite(cos)
+        return 1/sin(arg).rewrite(cos)
 
     def _eval_rewrite_as_sec(self, arg, **kwargs):
-        return sec(pi / 2 - arg, evaluate=False)
+        return sec(pi/2 - arg, evaluate=False)
 
     def _eval_rewrite_as_tan(self, arg, **kwargs):
-        return (1 / sin(arg).rewrite(tan))
+        return (1/sin(arg).rewrite(tan))
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -1845,7 +1844,7 @@ class csc(ReciprocalTrigonometricFunction):
 
     def _eval_is_complex(self):
         arg = self.args[0]
-        if arg.is_real and (arg / pi).is_integer is False:
+        if arg.is_real and (arg/pi).is_integer is False:
             return True
 
     @staticmethod
@@ -1941,7 +1940,7 @@ class sinc(Function):
                 if fuzzy_not(arg.is_zero):
                     return S.Zero
             elif (2*pi_coeff).is_integer:
-                return S.NegativeOne**(pi_coeff - S.Half) / arg
+                return S.NegativeOne**(pi_coeff - S.Half)/arg
 
     def _eval_nseries(self, x, n, logx):
         x = self.args[0]
@@ -2101,15 +2100,15 @@ class asin(InverseTrigonometricFunction):
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Infinity:
-                return S.NegativeInfinity * S.ImaginaryUnit
+                return S.NegativeInfinity*S.ImaginaryUnit
             elif arg is S.NegativeInfinity:
-                return S.Infinity * S.ImaginaryUnit
+                return S.Infinity*S.ImaginaryUnit
             elif arg.is_zero:
                 return S.Zero
             elif arg is S.One:
-                return S.Pi / 2
+                return S.Pi/2
             elif arg is S.NegativeOne:
-                return -S.Pi / 2
+                return -S.Pi/2
 
         if arg is S.ComplexInfinity:
             return S.ComplexInfinity
@@ -2124,7 +2123,7 @@ class asin(InverseTrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return S.ImaginaryUnit * asinh(i_coeff)
+            return S.ImaginaryUnit*asinh(i_coeff)
 
         if arg.is_zero:
             return S.Zero
@@ -2158,12 +2157,12 @@ class asin(InverseTrigonometricFunction):
             x = sympify(x)
             if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return p * (n - 2)**2/(n*(n - 1)) * x**2
+                return p*(n - 2)**2/(n*(n - 1))*x**2
             else:
                 k = (n - 1) // 2
                 R = RisingFactorial(S.Half, k)
                 F = factorial(k)
-                return R / F * x**n / n
+                return R/F*x**n/n
 
     def _eval_as_leading_term(self, x):
         from sympy import Order
@@ -2267,11 +2266,11 @@ class acos(InverseTrigonometricFunction):
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Infinity:
-                return S.Infinity * S.ImaginaryUnit
+                return S.Infinity*S.ImaginaryUnit
             elif arg is S.NegativeInfinity:
-                return S.NegativeInfinity * S.ImaginaryUnit
+                return S.NegativeInfinity*S.ImaginaryUnit
             elif arg.is_zero:
-                return S.Pi / 2
+                return S.Pi/2
             elif arg is S.One:
                 return S.Zero
             elif arg is S.NegativeOne:
@@ -2309,19 +2308,19 @@ class acos(InverseTrigonometricFunction):
     @cacheit
     def taylor_term(n, x, *previous_terms):
         if n == 0:
-            return S.Pi / 2
+            return S.Pi/2
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
             x = sympify(x)
             if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return p * (n - 2)**2/(n*(n - 1)) * x**2
+                return p*(n - 2)**2/(n*(n - 1))*x**2
             else:
                 k = (n - 1) // 2
                 R = RisingFactorial(S.Half, k)
                 F = factorial(k)
-                return -R / F * x**n / n
+                return -R/F*x**n/n
 
     def _eval_as_leading_term(self, x):
         from sympy import Order
@@ -2343,8 +2342,8 @@ class acos(InverseTrigonometricFunction):
         return self._eval_rewrite_as_log(self.args[0])._eval_nseries(x, n, logx)
 
     def _eval_rewrite_as_log(self, x, **kwargs):
-        return S.Pi/2 + S.ImaginaryUnit * \
-            log(S.ImaginaryUnit * x + sqrt(1 - x**2))
+        return S.Pi/2 + S.ImaginaryUnit*\
+            log(S.ImaginaryUnit*x + sqrt(1 - x**2))
 
     def _eval_rewrite_as_asin(self, x, **kwargs):
         return S.Pi/2 - asin(x)
@@ -2447,15 +2446,15 @@ class atan(InverseTrigonometricFunction):
             if arg is S.NaN:
                 return S.NaN
             elif arg is S.Infinity:
-                return S.Pi / 2
+                return S.Pi/2
             elif arg is S.NegativeInfinity:
-                return -S.Pi / 2
+                return -S.Pi/2
             elif arg.is_zero:
                 return S.Zero
             elif arg is S.One:
-                return S.Pi / 4
+                return S.Pi/4
             elif arg is S.NegativeOne:
-                return -S.Pi / 4
+                return -S.Pi/4
 
         if arg is S.ComplexInfinity:
             from sympy.calculus.util import AccumBounds
@@ -2471,7 +2470,7 @@ class atan(InverseTrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return S.ImaginaryUnit * atanh(i_coeff)
+            return S.ImaginaryUnit*atanh(i_coeff)
 
         if arg.is_zero:
             return S.Zero
@@ -2500,7 +2499,7 @@ class atan(InverseTrigonometricFunction):
             return S.Zero
         else:
             x = sympify(x)
-            return (-1)**((n - 1)//2) * x**n / n
+            return (-1)**((n - 1)//2)*x**n/n
 
     def _eval_as_leading_term(self, x):
         from sympy import Order
@@ -2512,8 +2511,8 @@ class atan(InverseTrigonometricFunction):
             return self.func(arg)
 
     def _eval_rewrite_as_log(self, x, **kwargs):
-        return S.ImaginaryUnit/2 * (log(S.One - S.ImaginaryUnit * x)
-            - log(S.One + S.ImaginaryUnit * x))
+        return S.ImaginaryUnit/2*(log(S.One - S.ImaginaryUnit*x)
+            - log(S.One + S.ImaginaryUnit*x))
 
     def _eval_aseries(self, n, args0, x, logx):
         if args0[0] is S.Infinity:
@@ -2521,7 +2520,7 @@ class atan(InverseTrigonometricFunction):
         elif args0[0] is S.NegativeInfinity:
             return (-S.Pi/2 - atan(1/self.args[0]))._eval_nseries(x, n, logx)
         else:
-            return super(atan, self)._eval_aseries(n, args0, x, logx)
+            return super()._eval_aseries(n, args0, x, logx)
 
     def inverse(self, argindex=1):
         """
@@ -2590,7 +2589,7 @@ class acot(InverseTrigonometricFunction):
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return -1 / (1 + self.args[0]**2)
+            return -1/(1 + self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -2623,9 +2622,9 @@ class acot(InverseTrigonometricFunction):
             elif arg.is_zero:
                 return S.Pi/ 2
             elif arg is S.One:
-                return S.Pi / 4
+                return S.Pi/4
             elif arg is S.NegativeOne:
-                return -S.Pi / 4
+                return -S.Pi/4
 
         if arg is S.ComplexInfinity:
             return S.Zero
@@ -2643,7 +2642,7 @@ class acot(InverseTrigonometricFunction):
 
         i_coeff = arg.as_coefficient(S.ImaginaryUnit)
         if i_coeff is not None:
-            return -S.ImaginaryUnit * acoth(i_coeff)
+            return -S.ImaginaryUnit*acoth(i_coeff)
 
         if arg.is_zero:
             return S.Pi*S.Half
@@ -2668,12 +2667,12 @@ class acot(InverseTrigonometricFunction):
     @cacheit
     def taylor_term(n, x, *previous_terms):
         if n == 0:
-            return S.Pi / 2  # FIX THIS
+            return S.Pi/2  # FIX THIS
         elif n < 0 or n % 2 == 0:
             return S.Zero
         else:
             x = sympify(x)
-            return (-1)**((n + 1)//2) * x**n / n
+            return (-1)**((n + 1)//2)*x**n/n
 
     def _eval_as_leading_term(self, x):
         from sympy import Order
@@ -2693,7 +2692,7 @@ class acot(InverseTrigonometricFunction):
             return super(atan, self)._eval_aseries(n, args0, x, logx)
 
     def _eval_rewrite_as_log(self, x, **kwargs):
-        return S.ImaginaryUnit/2 * (log(1 - S.ImaginaryUnit/x)
+        return S.ImaginaryUnit/2*(log(1 - S.ImaginaryUnit/x)
             - log(1 + S.ImaginaryUnit/x))
 
     def inverse(self, argindex=1):
@@ -3030,7 +3029,7 @@ class atan2(InverseTrigonometricFunction):
     `\operatorname{atan}` function for the point `(x, y) = (-1, 1)`
 
     >>> from sympy import atan, S
-    >>> atan(S(1) / -1)
+    >>> atan(S(1)/-1)
     -pi/4
     >>> atan2(1, -1)
     3*pi/4
@@ -3092,12 +3091,12 @@ class atan2(InverseTrigonometricFunction):
 
         if x.is_extended_real and y.is_extended_real:
             if x.is_positive:
-                return atan(y / x)
+                return atan(y/x)
             elif x.is_negative:
                 if y.is_negative:
-                    return atan(y / x) - S.Pi
+                    return atan(y/x) - S.Pi
                 elif y.is_nonnegative:
-                    return atan(y / x) + S.Pi
+                    return atan(y/x) + S.Pi
             elif x.is_zero:
                 if y.is_positive:
                     return S.Pi/2
@@ -3107,7 +3106,7 @@ class atan2(InverseTrigonometricFunction):
                     return S.NaN
         if y.is_zero:
             if x.is_extended_nonzero:
-                return S.Pi * (S.One - Heaviside(x))
+                return S.Pi*(S.One - Heaviside(x))
             if x.is_number:
                 return Piecewise((S.Pi, re(x) < 0),
                                  (0, Ne(x, 0)),
@@ -3117,7 +3116,7 @@ class atan2(InverseTrigonometricFunction):
                 (x + S.ImaginaryUnit*y)/sqrt(x**2 + y**2))
 
     def _eval_rewrite_as_log(self, y, x, **kwargs):
-        return -S.ImaginaryUnit*log((x + S.ImaginaryUnit*y) / sqrt(x**2 + y**2))
+        return -S.ImaginaryUnit*log((x + S.ImaginaryUnit*y)/sqrt(x**2 + y**2))
 
     def _eval_rewrite_as_atan(self, y, x, **kwargs):
         from sympy import re
@@ -3154,4 +3153,4 @@ class atan2(InverseTrigonometricFunction):
     def _eval_evalf(self, prec):
         y, x = self.args
         if x.is_extended_real and y.is_extended_real:
-            return super(atan2, self)._eval_evalf(prec)
+            return super()._eval_evalf(prec)
