@@ -17,24 +17,19 @@ False
 
 """
 
-from __future__ import division, print_function
-
 import warnings
 
 from sympy.core import S, sympify, Expr
-from sympy.core.numbers import Number
-from sympy.core.compatibility import iterable, is_sequence, as_int
+from sympy.core.compatibility import is_sequence
 from sympy.core.containers import Tuple
 from sympy.simplify import nsimplify, simplify
 from sympy.geometry.exceptions import GeometryError
 from sympy.functions.elementary.miscellaneous import sqrt
 from sympy.functions.elementary.complexes import im
 from sympy.matrices import Matrix
-from sympy.core.relational import Eq
 from sympy.core.numbers import Float
-from sympy.core.evaluate import global_evaluate
+from sympy.core.parameters import global_parameters
 from sympy.core.add import Add
-from sympy.sets import FiniteSet
 from sympy.utilities.iterables import uniq
 from sympy.utilities.misc import filldedent, func_name, Undecidable
 
@@ -109,7 +104,7 @@ class Point(GeometryEntity):
     is_Point = True
 
     def __new__(cls, *args, **kwargs):
-        evaluate = kwargs.get('evaluate', global_evaluate[0])
+        evaluate = kwargs.get('evaluate', global_parameters.evaluate)
         on_morph = kwargs.get('on_morph', 'ignore')
 
         # unpack into coords
@@ -140,7 +135,7 @@ class Point(GeometryEntity):
                 Point requires 2 or more coordinates or
                 keyword `dim` > 1.'''))
         if len(coords) != dim:
-            message = ("Dimension of {} needs to be changed"
+            message = ("Dimension of {} needs to be changed "
                        "from {} to {}.").format(coords, len(coords), dim)
             if on_morph == 'ignore':
                 pass
@@ -152,7 +147,7 @@ class Point(GeometryEntity):
                 raise ValueError(filldedent('''
                         on_morph value should be 'error',
                         'warn' or 'ignore'.'''))
-        if any(i for i in coords[dim:]):
+        if any(coords[dim:]):
             raise ValueError('Nonzero coordinates cannot be removed.')
         if any(a.is_number and im(a) for a in coords):
             raise ValueError('Imaginary coordinates are not permitted.')
@@ -165,9 +160,9 @@ class Point(GeometryEntity):
         # Turn any Floats into rationals and simplify
         # any expressions before we instantiate
         if evaluate:
-            coords = coords.xreplace(dict(
-                [(f, simplify(nsimplify(f, rational=True)))
-                 for f in coords.atoms(Float)]))
+            coords = coords.xreplace({
+                f: simplify(nsimplify(f, rational=True))
+                 for f in coords.atoms(Float)})
 
         # return 2D or 3D instances
         if len(coords) == 2:
@@ -281,6 +276,10 @@ class Point(GeometryEntity):
         coords = [simplify(x*factor) for x in self.args]
         return Point(coords, evaluate=False)
 
+    def __rmul__(self, factor):
+        """Multiply a factor by point's coordinates."""
+        return self.__mul__(factor)
+
     def __neg__(self):
         """Negate the point."""
         coords = [-x for x in self.args]
@@ -327,7 +326,9 @@ class Point(GeometryEntity):
         points = [i - origin for i in points[1:]]
 
         m = Matrix([i.args for i in points])
-        return m.rank()
+        # XXX fragile -- what is a better way?
+        return m.rank(iszerofunc = lambda x:
+            abs(x.n(2)) < 1e-12 if x.is_number else x.is_zero)
 
     @property
     def ambient_dimension(self):
@@ -380,19 +381,20 @@ class Point(GeometryEntity):
         points = list(uniq(points))
         return Point.affine_rank(*points) <= 2
 
-    def distance(self, p):
-        """The Euclidean distance from self to point p.
-
-        Parameters
-        ==========
-
-        p : Point
+    def distance(self, other):
+        """The Euclidean distance between self and another GeometricEntity.
 
         Returns
         =======
 
         distance : number or symbolic expression.
 
+        Raises
+        ======
+
+        TypeError : if other is not recognized as a GeometricEntity or is a
+                    GeometricEntity for which distance is not defined.
+
         See Also
         ========
 
@@ -402,19 +404,34 @@ class Point(GeometryEntity):
         Examples
         ========
 
-        >>> from sympy.geometry import Point
+        >>> from sympy.geometry import Point, Line
         >>> p1, p2 = Point(1, 1), Point(4, 5)
+        >>> l = Line((3, 1), (2, 2))
         >>> p1.distance(p2)
         5
+        >>> p1.distance(l)
+        sqrt(2)
+
+        The computed distance may be symbolic, too:
 
         >>> from sympy.abc import x, y
         >>> p3 = Point(x, y)
-        >>> p3.distance(Point(0, 0))
+        >>> p3.distance((0, 0))
         sqrt(x**2 + y**2)
 
         """
-        s, p = Point._normalize_dimension(self, Point(p))
-        return sqrt(Add(*((a - b)**2 for a, b in zip(s, p))))
+        if not isinstance(other, GeometryEntity):
+            try:
+                other = Point(other, dim=self.ambient_dimension)
+            except TypeError:
+                raise TypeError("not recognized as a GeometricEntity: %s" % type(other))
+        if isinstance(other, Point):
+            s, p = Point._normalize_dimension(self, Point(other))
+            return sqrt(Add(*((a - b)**2 for a, b in zip(s, p))))
+        distance = getattr(other, 'distance', None)
+        if distance is None:
+            raise TypeError("distance between Point and %s is not defined" % type(other))
+        return distance(self)
 
     def dot(self, p):
         """Return dot product of self with another Point."""
@@ -427,7 +444,7 @@ class Point(GeometryEntity):
         # a point is equal to another point if all its components are equal
         if not isinstance(other, Point) or len(self) != len(other):
             return False
-        return all(a.equals(b) for a,b in zip(self, other))
+        return all(a.equals(b) for a, b in zip(self, other))
 
     def evalf(self, prec=None, **options):
         """Evaluate the coordinates of the point.
@@ -466,7 +483,7 @@ class Point(GeometryEntity):
         Parameters
         ==========
 
-        other : Point
+        other : GeometryEntity or sequence of coordinates
 
         Returns
         =======
@@ -703,9 +720,9 @@ class Point(GeometryEntity):
         """
         dim = self.ambient_dimension
         # if a coordinate is zero, we can put a 1 there and zeros elsewhere
-        if self[0] == S.Zero:
+        if self[0].is_zero:
             return Point([1] + (dim - 1)*[0])
-        if self[1] == S.Zero:
+        if self[1].is_zero:
             return Point([0,1] + (dim - 2)*[0])
         # if the first two coordinates aren't zero, we can create a non-zero
         # orthogonal vector by swapping them, negating one, and padding with zeros
@@ -801,11 +818,6 @@ class Point(GeometryEntity):
         distances to point p. The weight used is the sum of absolute values
         of the coordinates.
 
-        See Also
-        ========
-
-        sympy.geometry.point.Point.distance
-
         Examples
         ========
 
@@ -920,7 +932,7 @@ class Point2D(Point):
         See Also
         ========
 
-        rotate, scale
+        translate, scale
 
         Examples
         ========
@@ -981,19 +993,14 @@ class Point2D(Point):
 
         See Also
         ========
-        geometry.entity.rotate
-        geometry.entity.scale
-        geometry.entity.translate
+        sympy.geometry.point.Point2D.rotate
+        sympy.geometry.point.Point2D.scale
+        sympy.geometry.point.Point2D.translate
         """
-        try:
-            col, row = matrix.shape
-            valid_matrix = matrix.is_square and col == 3
-        except AttributeError:
-            # We hit this block if matrix argument is not actually a Matrix.
-            valid_matrix = False
-        if not valid_matrix:
-            raise ValueError("The argument to the transform function must be " \
-            + "a 3x3 matrix")
+        if not (matrix.is_Matrix and matrix.shape == (3, 3)):
+            raise ValueError("matrix must be a 3x3 matrix")
+
+        col, row = matrix.shape
         x, y = self.args
         return Point(*(Matrix(1, 3, [x, y, 1])*matrix).tolist()[0][:2])
 
@@ -1003,7 +1010,7 @@ class Point2D(Point):
         See Also
         ========
 
-        rotate, scale
+        sympy.geometry.point.Point2D.rotate, scale
 
         Examples
         ========
@@ -1020,6 +1027,21 @@ class Point2D(Point):
         """
         return Point(self.x + x, self.y + y)
 
+    @property
+    def coordinates(self):
+        """
+        Returns the two coordinates of the Point.
+
+        Examples
+        ========
+
+        >>> from sympy import Point2D
+        >>> p = Point2D(0, 1)
+        >>> p.coordinates
+        (0, 1)
+        """
+        return self.args
+
     @property
     def x(self):
         """
@@ -1194,12 +1216,12 @@ class Point3D(Point):
         return [(point.x - self.x),(point.y - self.y),(point.z - self.z)]
 
     def intersection(self, other):
-        """The intersection between this point and another point.
+        """The intersection between this point and another GeometryEntity.
 
         Parameters
         ==========
 
-        other : Point
+        other : GeometryEntity or sequence of coordinates
 
         Returns
         =======
@@ -1264,19 +1286,13 @@ class Point3D(Point):
 
         See Also
         ========
-        geometry.entity.rotate
-        geometry.entity.scale
-        geometry.entity.translate
+        sympy.geometry.point.Point3D.scale
+        sympy.geometry.point.Point3D.translate
         """
-        try:
-            col, row = matrix.shape
-            valid_matrix = matrix.is_square and col == 4
-        except AttributeError:
-            # We hit this block if matrix argument is not actually a Matrix.
-            valid_matrix = False
-        if not valid_matrix:
-            raise ValueError("The argument to the transform function must be " \
-            + "a 4x4 matrix")
+        if not (matrix.is_Matrix and matrix.shape == (4, 4)):
+            raise ValueError("matrix must be a 4x4 matrix")
+
+        col, row = matrix.shape
         from sympy.matrices.expressions import Transpose
         x, y, z = self.args
         m = Transpose(matrix)
@@ -1288,7 +1304,7 @@ class Point3D(Point):
         See Also
         ========
 
-        rotate, scale
+        scale
 
         Examples
         ========
@@ -1305,6 +1321,21 @@ class Point3D(Point):
         """
         return Point3D(self.x + x, self.y + y, self.z + z)
 
+    @property
+    def coordinates(self):
+        """
+        Returns the three coordinates of the Point.
+
+        Examples
+        ========
+
+        >>> from sympy import Point3D
+        >>> p = Point3D(0, 1, 2)
+        >>> p.coordinates
+        (0, 1, 2)
+        """
+        return self.args
+
     @property
     def x(self):
         """
