@@ -1,11 +1,12 @@
 from __future__ import print_function, division
 
-__all__ = ['Particle']
-
-from sympy import sympify
+from sympy.core.backend import sympify
 from sympy.physics.vector import Point
+
 from sympy.utilities.exceptions import SymPyDeprecationWarning
 
+__all__ = ['Particle']
+
 
 class Particle(object):
     """A particle.
@@ -50,7 +51,8 @@ class Particle(object):
     def __str__(self):
         return self._name
 
-    __repr__ = __str__
+    def __repr__(self):
+        return self.__str__()
 
     @property
     def mass(self):
@@ -61,22 +63,6 @@ class Particle(object):
     def mass(self, value):
         self._mass = sympify(value)
 
-    def get_mass(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "Particle.get_mass(self)",
-                useinstead="property sympy.physics.mechanics.Particle.mass",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.mass
-
-    def set_mass(self, value):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "Particle.set_mass(self, value)",
-                useinstead="property sympy.physics.mechanics.Particle.mass",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.mass = value
-
     @property
     def point(self):
         """Point of the particle."""
@@ -88,22 +74,6 @@ class Particle(object):
             raise TypeError("Particle point attribute must be a Point object.")
         self._point = p
 
-    def get_point(self):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics." +
-                    "Particle.get_point(self)",
-                useinstead="property sympy.physics.mechanics.Particle.point",
-                deprecated_since_version="1.0", issue=9800).warn()
-        return self.point
-
-    def set_point(self, p):
-        SymPyDeprecationWarning(
-                feature="Method sympy.physics.mechanics.Particle." +
-                    "set_point(self, p)",
-                useinstead="property sympy.physics.mechanics.Particle.point",
-                deprecated_since_version="1.0", issue=9800).warn()
-        self.point = p
-
     def linear_momentum(self, frame):
         """Linear momentum of the particle.
 
@@ -263,5 +233,28 @@ class Particle(object):
                     "Particle.set_potential_energy(self, scalar)",
                 useinstead="property sympy.physics.mechanics." +
                     "Particle.potential_energy",
-                deprecated_since_version="1.0", issue=9800).warn()
+                deprecated_since_version="1.5", issue=9800).warn()
         self.potential_energy = scalar
+
+    def parallel_axis(self, point, frame):
+        """Returns an inertia dyadic of the particle with respect to another
+        point and frame.
+
+        Parameters
+        ==========
+        point : sympy.physics.vector.Point
+            The point to express the inertia dyadic about.
+        frame : sympy.physics.vector.ReferenceFrame
+            The reference frame used to construct the dyadic.
+
+        Returns
+        =======
+        inertia : sympy.physics.vector.Dyadic
+            The inertia dyadic of the particle expressed about the provided
+            point and frame.
+
+        """
+        # circular import issue
+        from sympy.physics.mechanics import inertia_of_point_mass
+        return inertia_of_point_mass(self.mass, self.point.pos_from(point),
+                                     frame)
