@@ -2,10 +2,12 @@
 
 from __future__ import print_function, division
 
+from typing import Any, Dict
+
 from operator import add, mul, lt, le, gt, ge
 from types import GeneratorType
 
-from sympy.core.compatibility import is_sequence, reduce, string_types, range
+from sympy.core.compatibility import is_sequence, reduce
 from sympy.core.expr import Expr
 from sympy.core.numbers import igcd, oo
 from sympy.core.symbol import Symbol, symbols as _symbols
@@ -39,10 +41,8 @@ def ring(symbols, domain, order=lex):
 
     symbols : str
         Symbol/Expr or sequence of str, Symbol/Expr (non-empty)
-    domain : :class:
-        `Domain` or coercible
-    order : :class:, optional
-        `Order` or coercible, optional, defaults to ``lex``
+    domain : :class:`~.Domain` or coercible
+    order : :class:`~.MonomialOrder` or coercible, optional, defaults to ``lex``
 
     Examples
     ========
@@ -72,10 +72,8 @@ def xring(symbols, domain, order=lex):
 
     symbols : str
         Symbol/Expr or sequence of str, Symbol/Expr (non-empty)
-    domain : :class:
-        `Domain` or coercible
-    order : :class:, optional
-        `Order` or coercible, optional, defaults to ``lex``
+    domain : :class:`~.Domain` or coercible
+    order : :class:`~.MonomialOrder` or coercible, optional, defaults to ``lex``
 
     Examples
     ========
@@ -105,10 +103,8 @@ def vring(symbols, domain, order=lex):
 
     symbols : str
         Symbol/Expr or sequence of str, Symbol/Expr (non-empty)
-    domain : :class:
-        `Domain` or coercible
-    order : :class:, optional
-        `Order` or coercible, optional, defaults to ``lex``
+    domain : :class:`~.Domain` or coercible
+    order : :class:`~.MonomialOrder` or coercible, optional, defaults to ``lex``
 
     Examples
     ========
@@ -136,10 +132,9 @@ def sring(exprs, *symbols, **options):
     Parameters
     ==========
 
-    exprs : :class:
-        `Expr` or sequence of :class:`Expr` (sympifiable)
-    symbols : sequence of :class:`Symbol`/:class:`Expr`
-    options : keyword arguments understood by :class:`Options`
+    exprs : :class:`~.Expr` or sequence of :class:`~.Expr` (sympifiable)
+    symbols : sequence of :class:`~.Symbol`/:class:`~.Expr`
+    options : keyword arguments understood by :class:`~.Options`
 
     Examples
     ========
@@ -185,19 +180,19 @@ def sring(exprs, *symbols, **options):
         return (_ring, polys)
 
 def _parse_symbols(symbols):
-    if isinstance(symbols, string_types):
+    if isinstance(symbols, str):
         return _symbols(symbols, seq=True) if symbols else ()
     elif isinstance(symbols, Expr):
         return (symbols,)
     elif is_sequence(symbols):
-        if all(isinstance(s, string_types) for s in symbols):
+        if all(isinstance(s, str) for s in symbols):
             return _symbols(symbols)
         elif all(isinstance(s, Expr) for s in symbols):
             return symbols
 
     raise GeneratorsError("expected a string, Symbol or expression or a non-empty sequence of strings, Symbols or expressions")
 
-_ring_cache = {}
+_ring_cache = {}  # type: Dict[Any, Any]
 
 class PolyRing(DefaultPrinting, IPolys):
     """Multivariate distributed polynomial ring. """
@@ -340,7 +335,7 @@ class PolyRing(DefaultPrinting, IPolys):
                 return self.ground_new(element)
             else:
                 raise NotImplementedError("conversion")
-        elif isinstance(element, string_types):
+        elif isinstance(element, str):
             raise NotImplementedError("parsing")
         elif isinstance(element, dict):
             return self.from_dict(element)
@@ -423,7 +418,7 @@ class PolyRing(DefaultPrinting, IPolys):
                 i = self.gens.index(gen)
             except ValueError:
                 raise ValueError("invalid generator: %s" % gen)
-        elif isinstance(gen, string_types):
+        elif isinstance(gen, str):
             try:
                 i = self.symbols.index(gen)
             except ValueError:
@@ -694,8 +689,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
             for k in p1.keys():
                 if not almosteq(p1[k], p2[k], tolerance):
                     return False
-            else:
-                return True
+            return True
         elif len(p1) > 1:
             return False
         else:
@@ -1193,7 +1187,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
             return self.square()
         elif n == 3:
             return self*self.square()
-        elif len(self) <= 5: # TODO: use an actuall density measure
+        elif len(self) <= 5: # TODO: use an actual density measure
             return self._pow_multinomial(n)
         else:
             return self._pow_generic(n)
@@ -1806,7 +1800,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         Parameters
         ==========
 
-        order : :class:`Order` or coercible, optional
+        order : :class:`~.MonomialOrder` or coercible, optional
 
         Examples
         ========
@@ -1832,7 +1826,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         Parameters
         ==========
 
-        order : :class:`Order` or coercible, optional
+        order : :class:`~.MonomialOrder` or coercible, optional
 
         Examples
         ========
@@ -1858,7 +1852,7 @@ class PolyElement(DomainElement, DefaultPrinting, CantSympify, dict):
         Parameters
         ==========
 
-        order : :class:`Order` or coercible, optional
+        order : :class:`~.MonomialOrder` or coercible, optional
 
         Examples
         ========
