@@ -2,15 +2,13 @@
 
 from __future__ import print_function, division
 
+from sympy.polys.densearith import (
+    dup_neg, dup_rshift, dup_rem)
 from sympy.polys.densebasic import (
     dup_LC, dup_TC, dup_degree,
     dup_strip, dup_reverse,
     dup_convert,
     dup_terms_gcd)
-
-from sympy.polys.densearith import (
-    dup_neg, dup_rshift, dup_rem)
-
 from sympy.polys.densetools import (
     dup_clear_denoms,
     dup_mirror, dup_scale, dup_shift,
@@ -19,18 +17,13 @@ from sympy.polys.densetools import (
     dup_eval, dmp_eval_in,
     dup_sign_variations,
     dup_real_imag)
-
-from sympy.polys.sqfreetools import (
-    dup_sqf_part, dup_sqf_list)
-
 from sympy.polys.factortools import (
     dup_factor_list)
-
 from sympy.polys.polyerrors import (
     RefinementFailed,
     DomainError)
-
-from sympy.core.compatibility import range
+from sympy.polys.sqfreetools import (
+    dup_sqf_part, dup_sqf_list)
 
 
 def dup_sturm(f, K):
@@ -55,7 +48,7 @@ def dup_sturm(f, K):
     References
     ==========
 
-    1. [Davenport88]_
+    .. [1] [Davenport88]_
 
     """
     if not K.is_Field:
@@ -77,7 +70,7 @@ def dup_root_upper_bound(f, K):
 
     References
     ==========
-    .. [#] Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
+    .. [1] Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
         Values of the Positive Roots of Polynomials"
         Journal of Universal Computer Science, Vol. 15, No. 3, 523-537, 2009.
     """
@@ -121,9 +114,9 @@ def dup_root_lower_bound(f, K):
 
        References
        ==========
-       Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
-           Values of the Positive Roots of Polynomials"
-           Journal of Universal Computer Science, Vol. 15, No. 3, 523-537, 2009.
+       .. [1] Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
+              Values of the Positive Roots of Polynomials"
+              Journal of Universal Computer Science, Vol. 15, No. 3, 523-537, 2009.
     """
     bound = dup_root_upper_bound(dup_reverse(f), K)
 
@@ -492,11 +485,14 @@ def dup_isolate_real_roots_sqf(f, K, eps=None, inf=None, sup=None, fast=False, b
 
        References
        ==========
-       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
-       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
-       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
-       of the Continued Fractions Method Using New Bounds of Positive Roots.
-       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+       .. [1] Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative
+              Study of Two Real Root Isolation Methods. Nonlinear Analysis:
+              Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       .. [2] Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S.
+              Vigklas: Improving the Performance of the Continued Fractions
+              Method Using New Bounds of Positive Roots. Nonlinear Analysis:
+              Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+
     """
     if K.is_QQ:
         (_, f), K = dup_clear_denoms(f, K, convert=True), K.get_ring()
@@ -523,11 +519,15 @@ def dup_isolate_real_roots(f, K, eps=None, inf=None, sup=None, basis=False, fast
 
        References
        ==========
-       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
-       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
-       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
-       of the Continued Fractions Method Using New Bounds of Positive Roots.
-       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+
+       .. [1] Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative
+              Study of Two Real Root Isolation Methods. Nonlinear Analysis:
+              Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       .. [2] Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S.
+              Vigklas: Improving the Performance of the Continued Fractions
+              Method Using New Bounds of Positive Roots.
+              Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+
     """
     if K.is_QQ:
         (_, f), K = dup_clear_denoms(f, K, convert=True), K.get_ring()
@@ -560,11 +560,15 @@ def dup_isolate_real_roots_list(polys, K, eps=None, inf=None, sup=None, strict=F
 
        References
        ==========
-       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
-       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
-       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
-       of the Continued Fractions Method Using New Bounds of Positive Roots.
-       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+
+       .. [1] Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative
+              Study of Two Real Root Isolation Methods. Nonlinear Analysis:
+              Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       .. [2] Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S.
+              Vigklas: Improving the Performance of the Continued Fractions
+              Method Using New Bounds of Positive Roots.
+              Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+
     """
     if K.is_QQ:
         K, F, polys = K.get_ring(), K, polys[:]
@@ -1208,7 +1212,7 @@ def dup_count_complex_roots(f, K, inf=None, sup=None, exclude=None):
     f = dup_convert(f, K, F)
 
     if inf is None or sup is None:
-        n, lc = dup_degree(f), abs(dup_LC(f, F))
+        _, lc = dup_degree(f), abs(dup_LC(f, F))
         B = 2*max([ F.quo(abs(c), lc) for c in f ])
 
     if inf is None:
@@ -1511,13 +1515,13 @@ def dup_isolate_complex_roots_sqf(f, K, eps=None, inf=None, sup=None, blackbox=F
         return []
 
     if K.is_ZZ:
-        R, F = K, K.get_field()
+        F = K.get_field()
     else:
-        R, F = K.get_ring(), K
+        F = K
 
     f = dup_convert(f, K, F)
 
-    n, lc = dup_degree(f), abs(dup_LC(f, F))
+    lc = abs(dup_LC(f, F))
     B = 2*max([ F.quo(abs(c), lc) for c in f ])
 
     (u, v), (s, t) = (-B, F.zero), (B, B)
