@@ -5,37 +5,29 @@ from __future__ import print_function, division
 from sympy.core import (S, Expr, Integer, Float, I, oo, Add, Lambda,
     symbols, sympify, Rational, Dummy)
 from sympy.core.cache import cacheit
-from sympy.core.function import AppliedUndef
-from sympy.functions.elementary.miscellaneous import root as _root
-
-from sympy.polys.polytools import Poly, PurePoly, factor
-from sympy.polys.rationaltools import together
-from sympy.polys.polyfuncs import symmetrize, viete
-
-from sympy.polys.rootisolation import (
-    dup_isolate_complex_roots_sqf,
-    dup_isolate_real_roots_sqf)
-
-from sympy.polys.polyroots import (
-    roots_linear, roots_quadratic, roots_binomial,
-    preprocess_roots, roots)
-
+from sympy.core.compatibility import ordered
+from sympy.polys.domains import QQ
 from sympy.polys.polyerrors import (
     MultivariatePolynomialError,
     GeneratorsNeeded,
     PolynomialError,
     DomainError)
-
-from sympy.polys.domains import QQ
+from sympy.polys.polyfuncs import symmetrize, viete
+from sympy.polys.polyroots import (
+    roots_linear, roots_quadratic, roots_binomial,
+    preprocess_roots, roots)
+from sympy.polys.polytools import Poly, PurePoly, factor
+from sympy.polys.rationaltools import together
+from sympy.polys.rootisolation import (
+    dup_isolate_complex_roots_sqf,
+    dup_isolate_real_roots_sqf)
+from sympy.utilities import lambdify, public, sift, numbered_symbols
 
 from mpmath import mpf, mpc, findroot, workprec
 from mpmath.libmp.libmpf import dps_to_prec, prec_to_dps
 
-from sympy.utilities import lambdify, public, sift
+from itertools import chain
 
-from sympy.core.compatibility import range, ordered
-
-from math import log as mathlog
 
 __all__ = ['CRootOf']
 
@@ -136,7 +128,7 @@ def _imag_count_of_factor(f):
 def rootof(f, x, index=None, radicals=True, expand=True):
     """An indexed root of a univariate polynomial.
 
-    Returns either a ``ComplexRootOf`` object or an explicit
+    Returns either a :obj:`ComplexRootOf` object or an explicit
     expression involving radicals.
 
     Parameters
@@ -163,7 +155,7 @@ class RootOf(Expr):
     Only complex roots are currently supported.
     """
 
-    __slots__ = ['poly']
+    __slots__ = ('poly',)
 
     def __new__(cls, f, x, index=None, radicals=True, expand=True):
         """Construct a new ``CRootOf`` object for ``k``-th root of ``f``."""
@@ -176,7 +168,8 @@ class ComplexRootOf(RootOf):
     Roots of a univariate polynomial separated into disjoint
     real or complex intervals and indexed in a fixed order.
     Currently only rational coefficients are allowed.
-    Can be imported as ``CRootOf``.
+    Can be imported as ``CRootOf``. To avoid confusion, the
+    generator must be a Symbol.
 
 
     Examples
@@ -188,6 +181,7 @@ class ComplexRootOf(RootOf):
     CRootOf is a way to reference a particular root of a
     polynomial. If there is a rational root, it will be returned:
 
+    >>> CRootOf.clear_cache()  # for doctest reproducibility
     >>> CRootOf(x**2 - 4, 0)
     -2
 
@@ -229,17 +223,17 @@ class ComplexRootOf(RootOf):
 
     After evaluation it is now
 
-    >>. r._get_interval()
+    >>> r._get_interval() # doctest: +SKIP
     (-165/169, -206/211)
 
-    To reset all intervals for a given polynomial, the `_reset` method
+    To reset all intervals for a given polynomial, the :meth:`_reset` method
     can be called from any CRootOf instance of the polynomial:
 
     >>> r._reset()
     >>> r._get_interval()
     (-1, 0)
 
-    The `eval_approx` method will also find the root to a given
+    The :meth:`eval_approx` method will also find the root to a given
     precision but the interval is not modified unless the search
     for the root fails to converge within the root bounds. And
     the secant method is used to find the root. (The ``evalf``
@@ -269,16 +263,35 @@ class ComplexRootOf(RootOf):
     >>> t.eval_rational(n=2)
     104755/2097152 - 6634255*I/2097152
 
+    Notes
+    =====
+
+    Although a PurePoly can be constructed from a non-symbol generator
+    RootOf instances of non-symbols are disallowed to avoid confusion
+    over what root is being represented.
+
+    >>> from sympy import exp, PurePoly
+    >>> PurePoly(x) == PurePoly(exp(x))
+    True
+    >>> CRootOf(x - 1, 0)
+    1
+    >>> CRootOf(exp(x) - 1, 0)  # would correspond to x == 0
+    Traceback (most recent call last):
+    ...
+    sympy.polys.polyerrors.PolynomialError: generator must be a Symbol
+
     See Also
     ========
+
     eval_approx
     eval_rational
-    _eval_evalf
+
     """
 
-    __slots__ = ['index']
+    __slots__ = ('index',)
     is_complex = True
     is_number = True
+    is_finite = True
 
     def __new__(cls, f, x, index=None, radicals=False, expand=True):
         """ Construct an indexed complex root of a polynomial.
@@ -305,6 +318,11 @@ class ComplexRootOf(RootOf):
         if not poly.is_univariate:
             raise PolynomialError("only univariate polynomials are allowed")
 
+        if not poly.gen.is_Symbol:
+            # PurePoly(sin(x) + 1) == PurePoly(x + 1) but the roots of
+            # x for each are not the same: issue 8617
+            raise PolynomialError("generator must be a Symbol")
+
         degree = poly.degree()
 
         if degree <= 0:
@@ -391,26 +409,26 @@ class ComplexRootOf(RootOf):
         return cls._get_roots("_all_roots", poly, radicals)
 
     @classmethod
-    def _get_reals_sqf(cls, factor, use_cache=True):
+    def _get_reals_sqf(cls, currentfactor, use_cache=True):
         """Get real root isolating intervals for a square-free factor."""
-        if use_cache and factor in _reals_cache:
-            real_part = _reals_cache[factor]
+        if use_cache and currentfactor in _reals_cache:
+            real_part = _reals_cache[currentfactor]
         else:
-            _reals_cache[factor] = real_part = \
+            _reals_cache[currentfactor] = real_part = \
                 dup_isolate_real_roots_sqf(
-                    factor.rep.rep, factor.rep.dom, blackbox=True)
+                    currentfactor.rep.rep, currentfactor.rep.dom, blackbox=True)
 
         return real_part
 
     @classmethod
-    def _get_complexes_sqf(cls, factor, use_cache=True):
+    def _get_complexes_sqf(cls, currentfactor, use_cache=True):
         """Get complex root isolating intervals for a square-free factor."""
-        if use_cache and factor in _complexes_cache:
-            complex_part = _complexes_cache[factor]
+        if use_cache and currentfactor in _complexes_cache:
+            complex_part = _complexes_cache[currentfactor]
         else:
-            _complexes_cache[factor] = complex_part = \
+            _complexes_cache[currentfactor] = complex_part = \
                 dup_isolate_complex_roots_sqf(
-                factor.rep.rep, factor.rep.dom, blackbox=True)
+                currentfactor.rep.rep, currentfactor.rep.dom, blackbox=True)
         return complex_part
 
     @classmethod
@@ -418,15 +436,15 @@ class ComplexRootOf(RootOf):
         """Compute real root isolating intervals for a list of factors. """
         reals = []
 
-        for factor, k in factors:
+        for currentfactor, k in factors:
             try:
                 if not use_cache:
                     raise KeyError
-                r = _reals_cache[factor]
-                reals.extend([(i, factor, k) for i in r])
+                r = _reals_cache[currentfactor]
+                reals.extend([(i, currentfactor, k) for i in r])
             except KeyError:
-                real_part = cls._get_reals_sqf(factor, use_cache)
-                new = [(root, factor, k) for root in real_part]
+                real_part = cls._get_reals_sqf(currentfactor, use_cache)
+                new = [(root, currentfactor, k) for root in real_part]
                 reals.extend(new)
 
         reals = cls._reals_sorted(reals)
@@ -437,15 +455,15 @@ class ComplexRootOf(RootOf):
         """Compute complex root isolating intervals for a list of factors. """
         complexes = []
 
-        for factor, k in ordered(factors):
+        for currentfactor, k in ordered(factors):
             try:
                 if not use_cache:
                     raise KeyError
-                c = _complexes_cache[factor]
-                complexes.extend([(i, factor, k) for i in c])
+                c = _complexes_cache[currentfactor]
+                complexes.extend([(i, currentfactor, k) for i in c])
             except KeyError:
-                complex_part = cls._get_complexes_sqf(factor, use_cache)
-                new = [(root, factor, k) for root in complex_part]
+                complex_part = cls._get_complexes_sqf(currentfactor, use_cache)
+                new = [(root, currentfactor, k) for root in complex_part]
                 complexes.extend(new)
 
         complexes = cls._complexes_sorted(complexes)
@@ -465,14 +483,14 @@ class ComplexRootOf(RootOf):
 
         reals = sorted(reals, key=lambda r: r[0].a)
 
-        for root, factor, _ in reals:
-            if factor in cache:
-                cache[factor].append(root)
+        for root, currentfactor, _ in reals:
+            if currentfactor in cache:
+                cache[currentfactor].append(root)
             else:
-                cache[factor] = [root]
+                cache[currentfactor] = [root]
 
-        for factor, roots in cache.items():
-            _reals_cache[factor] = roots
+        for currentfactor, root in cache.items():
+            _reals_cache[currentfactor] = root
 
         return reals
 
@@ -557,11 +575,11 @@ class ComplexRootOf(RootOf):
         # update cache
         cache = {}
         # -- collate
-        for root, factor, _ in complexes:
-            cache.setdefault(factor, []).append(root)
+        for root, currentfactor, _ in complexes:
+            cache.setdefault(currentfactor, []).append(root)
         # -- store
-        for factor, roots in cache.items():
-            _complexes_cache[factor] = roots
+        for currentfactor, root in cache.items():
+            _complexes_cache[currentfactor] = root
 
         return complexes
 
@@ -573,12 +591,12 @@ class ComplexRootOf(RootOf):
         """
         i = 0
 
-        for j, (_, factor, k) in enumerate(reals):
+        for j, (_, currentfactor, k) in enumerate(reals):
             if index < i + k:
-                poly, index = factor, 0
+                poly, index = currentfactor, 0
 
-                for _, factor, _ in reals[:j]:
-                    if factor == poly:
+                for _, currentfactor, _ in reals[:j]:
+                    if currentfactor == poly:
                         index += 1
 
                 return poly, index
@@ -592,12 +610,12 @@ class ComplexRootOf(RootOf):
         the root belongs.
         """
         i = 0
-        for j, (_, factor, k) in enumerate(complexes):
+        for j, (_, currentfactor, k) in enumerate(complexes):
             if index < i + k:
-                poly, index = factor, 0
+                poly, index = currentfactor, 0
 
-                for _, factor, _ in complexes[:j]:
-                    if factor == poly:
+                for _, currentfactor, _ in complexes[:j]:
+                    if currentfactor == poly:
                         index += 1
 
                 index += len(_reals_cache[poly])
@@ -641,6 +659,9 @@ class ComplexRootOf(RootOf):
         return roots
 
     def _reset(self):
+        """
+        Reset all intervals
+        """
         self._all_roots(self.poly, use_cache=False)
 
     @classmethod
@@ -714,15 +735,42 @@ class ComplexRootOf(RootOf):
         """Return postprocessed roots of specified kind. """
         if not poly.is_univariate:
             raise PolynomialError("only univariate polynomials are allowed")
-
+        # get rid of gen and it's free symbol
+        d = Dummy()
+        poly = poly.subs(poly.gen, d)
+        x = symbols('x')
+        # see what others are left and select x or a numbered x
+        # that doesn't clash
+        free_names = {str(i) for i in poly.free_symbols}
+        for x in chain((symbols('x'),), numbered_symbols('x')):
+            if x.name not in free_names:
+                poly = poly.xreplace({d: x})
+                break
         coeff, poly = cls._preprocess_roots(poly)
         roots = []
 
         for root in getattr(cls, method)(poly):
             roots.append(coeff*cls._postprocess_root(root, radicals))
-
         return roots
 
+    @classmethod
+    def clear_cache(cls):
+        """Reset cache for reals and complexes.
+
+        The intervals used to approximate a root instance are updated
+        as needed. When a request is made to see the intervals, the
+        most current values are shown. `clear_cache` will reset all
+        CRootOf instances back to their original state.
+
+        See Also
+        ========
+
+        _reset
+        """
+        global _reals_cache, _complexes_cache
+        _reals_cache = _pure_key_dict()
+        _complexes_cache = _pure_key_dict()
+
     def _get_interval(self):
         """Internal function for retrieving isolation interval from cache. """
         if self.is_real:
@@ -929,8 +977,8 @@ class ComplexRootOf(RootOf):
         # is_real value of the CRootOf instance.
         if type(self) == type(other):
             return sympify(self == other)
-        if not (other.is_number and not other.has(AppliedUndef)):
-            return S.false
+        if not other.is_number:
+            return None
         if not other.is_finite:
             return S.false
         z = self.expr.subs(self.expr.free_symbols.pop(), other).is_zero
@@ -962,7 +1010,7 @@ CRootOf = ComplexRootOf
 class RootSum(Expr):
     """Represents a sum of all roots of a univariate polynomial. """
 
-    __slots__ = ['poly', 'fun', 'auto']
+    __slots__ = ('poly', 'fun', 'auto')
 
     def __new__(cls, expr, func=None, x=None, auto=True, quadratic=False):
         """Construct a new ``RootSum`` instance of roots of a polynomial."""
@@ -975,10 +1023,7 @@ class RootSum(Expr):
         if func is None:
             func = Lambda(poly.gen, poly.gen)
         else:
-            try:
-                is_func = func.is_Function
-            except AttributeError:
-                is_func = False
+            is_func = getattr(func, 'is_Function', False)
 
             if is_func and 1 in func.nargs:
                 if not isinstance(func, Lambda):
